@@ -309,7 +309,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    *
    * Emitted when the button is pressed.
    * 
-   * @Deprecated: Use the GtkWidget::button-press-event signal.
+   * @Deprecated: Use the #GtkWidget::button-press-event signal.
    */ 
   button_signals[PRESSED] =
     g_signal_new (I_("pressed"),
@@ -326,7 +326,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    *
    * Emitted when the button is released.
    * 
-   * @Deprecated: Use the GtkWidget::button-release-event signal.
+   * @Deprecated: Use the #GtkWidget::button-release-event signal.
    */ 
   button_signals[RELEASED] =
     g_signal_new (I_("released"),
@@ -358,7 +358,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    *
    * Emitted when the pointer enters the button.
    * 
-   * @Deprecated: Use the GtkWidget::enter-notify-event signal.
+   * @Deprecated: Use the #GtkWidget::enter-notify-event signal.
    */ 
   button_signals[ENTER] =
     g_signal_new (I_("enter"),
@@ -375,7 +375,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    *
    * Emitted when the pointer leaves the button.
    * 
-   * @Deprecated: Use the GtkWidget::leave-notify-event signal.
+   * @Deprecated: Use the #GtkWidget::leave-notify-event signal.
    */ 
   button_signals[LEAVE] =
     g_signal_new (I_("leave"),
@@ -390,10 +390,10 @@ gtk_button_class_init (GtkButtonClass *klass)
    * GtkButton::activate:
    * @widget: the object which received the signal.
    *
-   * The "activate" signal on GtkButton is an action signal and
+   * The ::activate signal on GtkButton is an action signal and
    * emitting it causes the button to animate press then release. 
    * Applications should never connect to this signal, but use the
-   * "clicked" signal.
+   * #GtkButton::clicked signal.
    */
   button_signals[ACTIVATE] =
     g_signal_new (I_("activate"),
@@ -438,8 +438,8 @@ gtk_button_class_init (GtkButtonClass *klass)
   /**
    * GtkButton:displace-focus:
    *
-   * Whether the child_displacement_x/child_displacement_y properties should also 
-   * affect the focus rectangle.
+   * Whether the child_displacement_x/child_displacement_y properties 
+   * should also affect the focus rectangle.
    *
    * Since: 2.6
    */
@@ -733,14 +733,14 @@ gtk_button_construct_child (GtkButton *button)
       image = g_object_ref (priv->image);
       if (image->parent)
 	gtk_container_remove (GTK_CONTAINER (image->parent), image);
-
-      priv->image = NULL;
     }
   
+  priv->image = NULL;
+
   if (GTK_BIN (button)->child)
     gtk_container_remove (GTK_CONTAINER (button),
 			  GTK_BIN (button)->child);
-  
+
   if (button->use_stock &&
       button->label_text &&
       gtk_stock_lookup (button->label_text, &item))
@@ -1057,7 +1057,7 @@ gtk_button_get_props (GtkButton *button,
       if (tmp_border)
 	{
 	  *default_border = *tmp_border;
-	  g_free (tmp_border);
+	  gtk_border_free (tmp_border);
 	}
       else
 	*default_border = default_default_border;
@@ -1070,7 +1070,7 @@ gtk_button_get_props (GtkButton *button,
       if (tmp_border)
 	{
 	  *default_outside_border = *tmp_border;
-	  g_free (tmp_border);
+	  gtk_border_free (tmp_border);
 	}
       else
 	*default_outside_border = default_default_outside_border;
@@ -1083,7 +1083,7 @@ gtk_button_get_props (GtkButton *button,
       if (tmp_border)
 	{
 	  *inner_border = *tmp_border;
-	  g_free (tmp_border);
+	  gtk_border_free (tmp_border);
 	}
       else
 	*inner_border = default_inner_border;
@@ -1490,12 +1490,8 @@ gtk_real_button_released (GtkButton *button)
 static gboolean
 button_activate_timeout (gpointer data)
 {
-  GDK_THREADS_ENTER ();
-  
   gtk_button_finish_activate (data, TRUE);
 
-  GDK_THREADS_LEAVE ();
-
   return FALSE;
 }
 
@@ -1520,7 +1516,7 @@ gtk_real_button_activate (GtkButton *button)
 
       gtk_grab_add (widget);
       
-      button->activate_timeout = g_timeout_add (ACTIVATE_TIMEOUT,
+      button->activate_timeout = gdk_threads_add_timeout (ACTIVATE_TIMEOUT,
 						button_activate_timeout,
 						button);
       button->button_down = TRUE;
@@ -1655,7 +1651,7 @@ gtk_button_get_use_underline (GtkButton *button)
  * @button: a #GtkButton
  * @use_stock: %TRUE if the button should use a stock item
  *
- * If true, the label set on the button is used as a
+ * If %TRUE, the label set on the button is used as a
  * stock id to select the stock item for the button.
  */
 void
@@ -1910,7 +1906,7 @@ gtk_button_screen_changed (GtkWidget *widget,
 
   show_image_connection =
     g_signal_connect (settings, "notify::gtk-button-images",
-		      G_CALLBACK (gtk_button_setting_changed), 0);
+		      G_CALLBACK (gtk_button_setting_changed), NULL);
   g_object_set_data (G_OBJECT (settings), 
 		     I_("gtk-button-connection"),
 		     GUINT_TO_POINTER (show_image_connection));
@@ -1957,7 +1953,7 @@ gtk_button_grab_notify (GtkWidget *widget,
  * @image: a widget to set as the image for the button
  *
  * Set the image of @button to the given widget. Note that
- * it depends on the gtk-button-images setting whether the
+ * it depends on the #GtkSettings:gtk-button-images setting whether the
  * image will be displayed or not, you don't have to call
  * gtk_widget_show() on @image yourself.
  *
