@@ -83,6 +83,8 @@ static void     gtk_paned_realize               (GtkWidget        *widget);
 static void     gtk_paned_unrealize             (GtkWidget        *widget);
 static void     gtk_paned_map                   (GtkWidget        *widget);
 static void     gtk_paned_unmap                 (GtkWidget        *widget);
+static void     gtk_paned_state_changed         (GtkWidget        *widget,
+                                                 GtkStateType      previous_state);
 static gboolean gtk_paned_expose                (GtkWidget        *widget,
 						 GdkEventExpose   *event);
 static gboolean gtk_paned_enter                 (GtkWidget        *widget,
@@ -194,6 +196,7 @@ gtk_paned_class_init (GtkPanedClass *class)
   widget_class->motion_notify_event = gtk_paned_motion;
   widget_class->grab_broken_event = gtk_paned_grab_broken;
   widget_class->grab_notify = gtk_paned_grab_notify;
+  widget_class->state_changed = gtk_paned_state_changed;
   
   container_class->add = gtk_paned_add;
   container_class->remove = gtk_paned_remove;
@@ -303,6 +306,19 @@ gtk_paned_class_init (GtkPanedClass *class)
 								    TRUE,
 								    GTK_PARAM_READWRITE));
 
+  /**
+   * GtkPaned::cycle-child-focus:
+   * @widget: the object that received the signal
+   * @reversed: whether cycling backward or forward
+   *
+   * The ::cycle-child-focus signal is a 
+   * <link linkend="keybinding-signals">keybinding signal</link>
+   * which gets emitted to cycle the focus between the children of the paned.
+   *
+   * The default binding is f6.
+   *
+   * Since: 2.0
+   */
   signals [CYCLE_CHILD_FOCUS] =
     g_signal_new (I_("cycle_child_focus"),
 		  G_TYPE_FROM_CLASS (object_class),
@@ -313,6 +329,19 @@ gtk_paned_class_init (GtkPanedClass *class)
 		  G_TYPE_BOOLEAN, 1,
 		  G_TYPE_BOOLEAN);
 
+  /**
+   * GtkPaned::toggle-handle-focus:
+   * @widget: the object that received the signal
+   *
+   * The ::toggle-handle-focus is a 
+   * <link linkend="keybinding-signals">keybinding signal</link>
+   * which gets emitted to accept the current position of the handle and then 
+   * move focus to the next widget in the focus chain.
+   *
+   * The default binding is Tab.
+   *
+   * Since: 2.0
+   */
   signals [TOGGLE_HANDLE_FOCUS] =
     g_signal_new (I_("toggle_handle_focus"),
 		  G_TYPE_FROM_CLASS (object_class),
@@ -322,6 +351,18 @@ gtk_paned_class_init (GtkPanedClass *class)
 		  _gtk_marshal_BOOLEAN__VOID,
 		  G_TYPE_BOOLEAN, 0);
 
+  /**
+   * GtkPaned::move-handle:
+   * @widget: the object that received the signal
+   * @scroll_type: a #GtkScrollType
+   *
+   * The ::move-handle signal is a 
+   * <link linkend="keybinding-signals">keybinding signal</link>
+   * which gets emitted to move the handle when the user is using key bindings 
+   * to move it.
+   *
+   * Since: 2.0
+   */
   signals[MOVE_HANDLE] =
     g_signal_new (I_("move_handle"),
 		  G_TYPE_FROM_CLASS (object_class),
@@ -332,6 +373,20 @@ gtk_paned_class_init (GtkPanedClass *class)
                   G_TYPE_BOOLEAN, 1,
                   GTK_TYPE_SCROLL_TYPE);
 
+  /**
+   * GtkPaned::cycle-handle-focus:
+   * @widget: the object that received the signal
+   * @reversed: whether cycling backward or forward
+   *
+   * The ::cycle-handle-focus signal is a 
+   * <link linkend="keybinding-signals">keybinding signal</link>
+   * which gets emitted to cycle whether the paned should grab focus to allow
+   * the user to change position of the handle by using key bindings.
+   *
+   * The default binding for this signal is f8.
+   *
+   * Since: 2.0
+   */
   signals [CYCLE_HANDLE_FOCUS] =
     g_signal_new (I_("cycle_handle_focus"),
 		  G_TYPE_FROM_CLASS (object_class),
@@ -342,6 +397,19 @@ gtk_paned_class_init (GtkPanedClass *class)
 		  G_TYPE_BOOLEAN, 1,
 		  G_TYPE_BOOLEAN);
 
+  /**
+   * GtkPaned::accept-position:
+   * @widget: the object that received the signal
+   *
+   * The ::accept-position signal is a 
+   * <link linkend="keybinding-signals">keybinding signal</link>
+   * which gets emitted to accept the current position of the handle when 
+   * moving it using key bindings.
+   *
+   * The default binding for this signal is Return or Space.
+   *
+   * Since: 2.0
+   */
   signals [ACCEPT_POSITION] =
     g_signal_new (I_("accept_position"),
 		  G_TYPE_FROM_CLASS (object_class),
@@ -351,6 +419,20 @@ gtk_paned_class_init (GtkPanedClass *class)
 		  _gtk_marshal_BOOLEAN__VOID,
 		  G_TYPE_BOOLEAN, 0);
 
+  /**
+   * GtkPaned::cancel-position:
+   * @widget: the object that received the signal
+   *
+   * The ::cancel-position signal is a 
+   * <link linkend="keybinding-signals">keybinding signal</link>
+   * which gets emitted to cancel moving the position of the handle using key 
+   * bindings. The position of the handle will be reset to the value prior to 
+   * moving it.
+   *
+   * The default binding for this signal is Escape.
+   *
+   * Since: 2.0
+   */
   signals [CANCEL_POSITION] =
     g_signal_new (I_("cancel_position"),
 		  G_TYPE_FROM_CLASS (object_class),
@@ -638,8 +720,6 @@ gtk_paned_realize (GtkWidget *widget)
   attributes.y = paned->handle_pos.y;
   attributes.width = paned->handle_pos.width;
   attributes.height = paned->handle_pos.height;
-  attributes.cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
-						  paned->cursor_type);
   attributes.event_mask = gtk_widget_get_events (widget);
   attributes.event_mask |= (GDK_BUTTON_PRESS_MASK |
 			    GDK_BUTTON_RELEASE_MASK |
@@ -647,12 +727,19 @@ gtk_paned_realize (GtkWidget *widget)
 			    GDK_LEAVE_NOTIFY_MASK |
 			    GDK_POINTER_MOTION_MASK |
 			    GDK_POINTER_MOTION_HINT_MASK);
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_CURSOR;
+  attributes_mask = GDK_WA_X | GDK_WA_Y;
+  if (GTK_WIDGET_IS_SENSITIVE (widget))
+    {
+      attributes.cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+						      paned->cursor_type);
+      attributes_mask |= GDK_WA_CURSOR;
+    }
 
   paned->handle = gdk_window_new (widget->window,
 				  &attributes, attributes_mask);
   gdk_window_set_user_data (paned->handle, paned);
-  gdk_cursor_unref (attributes.cursor);
+  if (attributes_mask & GDK_WA_CURSOR)
+    gdk_cursor_unref (attributes.cursor);
 
   widget->style = gtk_style_attach (widget->style, widget->window);
 
@@ -916,6 +1003,28 @@ gtk_paned_grab_notify (GtkWidget *widget,
     stop_drag (paned);
 }
 
+static void
+gtk_paned_state_changed (GtkWidget    *widget,
+                         GtkStateType  previous_state)
+{
+  GtkPaned *paned = GTK_PANED (widget);
+  GdkCursor *cursor;
+
+  if (GTK_WIDGET_REALIZED (paned))
+    {
+      if (GTK_WIDGET_IS_SENSITIVE (widget))
+        cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+                                             paned->cursor_type); 
+      else
+        cursor = NULL;
+
+      gdk_window_set_cursor (paned->handle, cursor);
+
+      if (cursor)
+        gdk_cursor_unref (cursor);
+    }
+}
+
 static gboolean
 gtk_paned_button_release (GtkWidget      *widget,
 			  GdkEventButton *event)
@@ -1123,6 +1232,11 @@ gtk_paned_set_position (GtkPaned *paned,
   g_object_thaw_notify (object);
 
   gtk_widget_queue_resize (GTK_WIDGET (paned));
+
+#ifdef G_OS_WIN32
+  /* Hacky work-around for bug #144269 */
+  gtk_widget_queue_resize (paned->child2);
+#endif
 }
 
 /**
@@ -1213,8 +1327,11 @@ gtk_paned_compute_position (GtkPaned *paned,
 			      paned->min_position,
 			      paned->max_position);
 
-  gtk_widget_set_child_visible (paned->child1, paned->child1_size != 0);
-  gtk_widget_set_child_visible (paned->child2, paned->child1_size != allocation);
+  if (paned->child1)
+    gtk_widget_set_child_visible (paned->child1, paned->child1_size != 0);
+  
+  if (paned->child2)
+    gtk_widget_set_child_visible (paned->child2, paned->child1_size != allocation); 
 
   g_object_freeze_notify (G_OBJECT (paned));
   if (paned->child1_size != old_position)
@@ -1483,7 +1600,7 @@ get_child_panes (GtkWidget  *widget,
     }
   else if (GTK_IS_CONTAINER (widget))
     {
-      gtk_container_foreach (GTK_CONTAINER (widget),
+      gtk_container_forall (GTK_CONTAINER (widget),
 			     (GtkCallback)get_child_panes, panes);
     }
 }
