@@ -530,10 +530,11 @@ gtk_target_list_remove (GtkTargetList *list,
  * gtk_target_list_find:
  * @list: a #GtkTargetList
  * @target: an interned atom representing the target to search for
- * @info: a pointer to the location to store application info for target
- * 
+ * @info: a pointer to the location to store application info for target,
+ *        or %NULL
+ *
  * Looks up a given target in a #GtkTargetList.
- * 
+ *
  * Return value: %TRUE if the target was found, otherwise %FALSE
  **/
 gboolean
@@ -541,16 +542,23 @@ gtk_target_list_find (GtkTargetList *list,
 		      GdkAtom        target,
 		      guint         *info)
 {
-  GList *tmp_list = list->list;
+  GList *tmp_list;
+
+  g_return_val_if_fail (list != NULL, FALSE);
+
+  tmp_list = list->list;
   while (tmp_list)
     {
       GtkTargetPair *pair = tmp_list->data;
 
       if (pair->target == target)
 	{
-	  *info = pair->info;
+          if (info)
+            *info = pair->info;
+
 	  return TRUE;
 	}
+
       tmp_list = tmp_list->next;
     }
 
@@ -1197,8 +1205,9 @@ normalize_to_crlf (const gchar *str,
 {
   GString *result = g_string_sized_new (len);
   const gchar *p = str;
+  const gchar *end = str + len;
 
-  while (1)
+  while (p < end)
     {
       if (*p == '\n')
 	g_string_append_c (result, '\r');
@@ -1207,13 +1216,12 @@ normalize_to_crlf (const gchar *str,
 	{
 	  g_string_append_c (result, *p);
 	  p++;
-	  if (*p != '\n')
+	  if (p == end || *p != '\n')
 	    g_string_append_c (result, '\n');
+	  if (p == end)
+	    break;
 	}
 
-      if (*p == '\0')
-	break;
-
       g_string_append_c (result, *p);
       p++;
     }
